#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "NebulaClient.h"
#include "IOTCClient.h"
#include "AVAPIs.h"
#include "disposable_params.h"
#include <windows.h>

static char gUdid[MAX_PUBLIC_UDID_LENGTH + 1] = {0};
static char gPincode[MAX_PIN_CODE_LENGTH+1] = {0};
static char gRentAmToken[MAX_TOKEN_LENGTH + 1] = {0};
static char gRealm[MAX_REALM_LENGTH + 1] = "hausetopia-us";

static unsigned int gBindAbort = 0;
static unsigned int gSendCommandAbort = 0;
static unsigned int gConnectAbort = 0;


int main(int argc, char *argv[])
{
	int ret;
    TUTKJsonObject *json_response = NULL;
    const char *profile_str = NULL;

	if (argc != 4)
	{
		printf("%s <LicenseKey> <UDID> <AM Token>\n",argv[0]);
		return -1;
	}

    char *license_key = argv[1];
    ret = TUTK_SDK_Set_License_Key(license_key);
    printf("TUTK_SDK_Set_License_Key: %d\n", ret);
    if (ret < 0) {
        return -1;
    }
    strcpy(gUdid, argv[2]);
    strcpy(gRentAmToken, argv[3]);

    ret = loadDisposableParams(gPincode, NULL, NULL);
    if (ret < 0) {
        printf("loadDisposableParams()=%d\n", ret);
        return -1;
    }

    LogAttr log_attr;
    memset(&log_attr, 0, sizeof(log_attr));
    char log_path[] = "./client_log.txt";
    log_attr.path = log_path;

    ret = IOTC_Set_Log_Attr(log_attr);
    printf("IOTC_Set_Log_Attr ret[%d]\n", ret);

	// use which Master base on location, port 0 means to get a random port
	ret = IOTC_Initialize2(0);
	printf("call IOTC_Initialize(), ret = %d\n", ret);
	if(ret != IOTC_ER_NoERROR)
	{
		printf("IOTCAPIs_Client exit...!!\n");
		return -1;
	}

    ret = Nebula_Initialize();
    printf("Nebula_Initialize ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Client exit...!!\n");
        return -1;
    }

    NebulaClientCtx *client_ctx;

    ret = Nebula_Client_New(gUdid, &client_ctx);
    printf("Nebula_Client_New ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Client exit...!!\n");
        return -1;
    }

    TUTKJsonObject *json_args = NULL;
    ret = Nebula_Client_Bind(client_ctx, gPincode, &json_args, 30000, &gBindAbort);
    printf("Nebula_Client_Bind ret[%d]\n", ret);

    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Client exit...!!\n");
        return -1;
    }

    profile_str = TUTK_Json_Obj_To_String(json_args);
    printf("[Bind_Response]\n%s\n", profile_str);

    const TUTKJsonObject *json_identity;
	TUTK_Json_Obj_Get_Sub_Obj(json_args, "identity", &json_identity);
    const char *identity_value = NULL;
	TUTK_Json_Obj_Get_String(json_identity, &identity_value);
    printf("Get identity: %s\n", identity_value);

    const TUTKJsonObject *json_avtoken;
	TUTK_Json_Obj_Get_Sub_Obj(json_args, "avToken", &json_avtoken);
    const char *avtoken_value = NULL;
	TUTK_Json_Obj_Get_String(json_avtoken, &avtoken_value);
    printf("Get avtoken: %s\n", avtoken_value);

    ret = Nebula_Client_Free_Bind_Response(json_args);
    if (ret != NEBULA_ER_NoERROR)
        printf("Nebula_Client_Free_Bind_Response failed!\n");

    char request1[] = "{\"func\":\"getProfile\"}";
    printf("get device's profile : \n%s\n", request1);
    ret = Nebula_Client_Send_Command(client_ctx, request1, &json_response, 10000, &gSendCommandAbort);
    printf("json_response [%d] \n\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Client exit...!!\n");
        return -1;
    }

    profile_str = TUTK_Json_Obj_To_String(json_response);
    printf("[getProfile]\n%s\n", profile_str);

    ret = Nebula_Client_Free_Send_Command_Response(json_response);
    if (ret != NEBULA_ER_NoERROR)
        printf("Nebula_Client_Free_Send_Command_Response failed!\n");
    

    char request2[] = "{\"func\":\"getNightMode\"}";
    printf("get device's night mode : \n%s\n", request2);
    ret = Nebula_Client_Send_Command(client_ctx, request2, &json_response, 10000, &gSendCommandAbort);
    printf("json_response [%d]\n\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Client exit...!!\n");
        return -1;
    }

    profile_str = TUTK_Json_Obj_To_String(json_response);
    printf("[getNightMode]\n%s\n", profile_str);

    ret = Nebula_Client_Free_Send_Command_Response(json_response);
    if (ret != NEBULA_ER_NoERROR)
        printf("Nebula_Client_Free_Send_Command_Response failed!\n");

    ret = IOTC_Client_Connect_By_Nebula(client_ctx, gRentAmToken, gRealm, 60000, &gConnectAbort);

    printf("IOTC_Client_Connect_By_Nebula ret[%d]\n", ret);
    if (ret >= IOTC_ER_NoERROR) {
        printf("Session[%d] connect!\n", ret);
        struct st_SInfoEx Sinfo;
        Sinfo.size = sizeof(struct st_SInfoEx);
        if(IOTC_Session_Check_Ex(ret, &Sinfo) == IOTC_ER_NoERROR)
        {
            char *mode[3] = {"P2P", "RLY", "LAN"};
            if( isdigit( Sinfo.RemoteIP[0] ))
                printf("device is from[IP:%s, Port:%d] Mode[%s] VPG[%d:%d:%d] VER[%X] NAT[%d] AES[%d]\n", Sinfo.RemoteIP, Sinfo.RemotePort, mode[(int)Sinfo.Mode], Sinfo.VID, Sinfo.PID, Sinfo.GID, Sinfo.IOTCVersion, Sinfo.RemoteNatType, Sinfo.isSecure);
        }
    }

    ret = Nebula_Client_Delete(client_ctx); 
    Nebula_DeInitialize();
	IOTC_DeInitialize();
    printf("IOTC_DeInitialize()\n");
	return 0;
}
